@@ -30,7 +30,7 @@
 
 // For pthread_setname_np() definition and various other needs:
 #ifndef _GNU_SOURCE
-#define _GNU_SOURCE
+#define _GNU_SOURCE 1
 #endif
 
 #if PSYCH_SYSTEM == PSYCH_WINDOWS
@@ -57,6 +57,13 @@
     #define printf mexPrintf
 #endif
 
+#if PSYCH_LANGUAGE == PSYCH_PYTHON
+    #undef printf
+    #define printf PySys_WriteStdout
+    #undef fprintf
+    #define fprintf(fdignore, ...) PySys_WriteStderr(__VA_ARGS__)
+#endif
+
 //platform dependent macro defines
 #if PSYCH_SYSTEM == PSYCH_WINDOWS
     // Define snprintf as _snprintf to take Microsoft brain-damage into account:
@@ -179,6 +186,13 @@ typedef unsigned char        psych_bool;
     typedef mxLogical PsychNativeBooleanType;
 #endif
 
+#if PSYCH_LANGUAGE == PSYCH_PYTHON
+    #define PsychGenericScriptType PyObject
+    typedef unsigned char PsychNativeBooleanType;
+    typedef size_t ptbSize;
+    typedef size_t ptbIndex;
+#endif
+
 #define EXP
 #ifndef APIENTRY
 #define APIENTRY
