@@ -68,8 +68,8 @@
 
 #include "PsychHID.h"
 
-static char useString[]= "PsychHID('KbQueueCreate', [deviceNumber][, keyFlags=all][, numValuators=0][, numSlots=10000][, flags=0])";
-//                                                   1               2               3                 4                 5
+static char useString[]= "PsychHID('KbQueueCreate', [deviceNumber][, keyFlags=all][, numValuators=0][, numSlots=10000][, flags=0][, windowHandle=0])";
+//                                                   1               2               3                 4                 5          6
 static char synopsisString[] =
 "Creates a queue for events generated by an input device (keyboard, keypad, mouse, ...).\n"
 "By default the default keyboard device (as defined by an operating system specific heuristic) is "
@@ -98,6 +98,8 @@ static char synopsisString[] =
 "+1 = Always label synthetic key repeat events as invalid - iow. suppress them. Supported on Linux only.\n"
 "+2 = Never label synthetic key repeat events as invalid - iow. accept them. Supported on Linux only.\n"
 "+4 = Try to get raw valuator state, avoiding things like pointer acceleration. Supported on Linux only.\n"
+"\n\n"
+"'windowHandle' Optional windowing system specific handle for an associated onscreen window. Used on Linux/X11 only.\n"
 "\n";
 
 static char seeAlsoString[] = "KbQueueStart, KbQueueStop, KbQueueCheck, KbQueueFlush, KbQueueRelease";
@@ -110,12 +112,13 @@ PsychError PSYCHHIDKbQueueCreate(void)
     int numValuators = 0;
     int numSlots = 10000;
     int flags = 0;
+    int windowHandle = 0;
     int rc;
 
     PsychPushHelp(useString, synopsisString, seeAlsoString);
     if(PsychIsGiveHelp()) { PsychGiveHelp(); return(PsychError_none);};
 
-    PsychErrorExit(PsychCapNumInputArgs(5));
+    PsychErrorExit(PsychCapNumInputArgs(6));
 
     // Get optional deviceIndex:
     PsychCopyInIntegerArg(1, FALSE, &deviceIndex);
@@ -138,8 +141,13 @@ PsychError PSYCHHIDKbQueueCreate(void)
     if (flags < 0)
         PsychErrorExitMsg(PsychError_user, "Invalid 'flags' provided. Must be at least 0.");
 
+    // Get optional window handle:
+    PsychCopyInIntegerArg(6, FALSE, &windowHandle);
+    if (windowHandle < 0)
+        PsychErrorExitMsg(PsychError_user, "Invalid 'windowHandle' provided. Must be at least 0.");
+
     // Perform actual, OS-dependent init and return its status code:
-    rc = PsychHIDOSKbQueueCreate(deviceIndex, numScankeys, scanKeys, numValuators, numSlots, (unsigned int) flags);
+    rc = PsychHIDOSKbQueueCreate(deviceIndex, numScankeys, scanKeys, numValuators, numSlots, (unsigned int) flags, (unsigned int) windowHandle);
 
     return(rc);
 }
