@@ -621,8 +621,6 @@ PsychError SCREENGetMouseHelper(void)
     double* buttonArray;
     PsychNativeBooleanType* buttonStates;
     int keysdown;
-    XEvent event_return;
-    XKeyPressedEvent keypressevent;
     int screenNumber;
     int priorityLevel;
     struct sched_param schedulingparam;
@@ -800,8 +798,22 @@ PsychError SCREENGetMouseHelper(void)
             if (indevs->use == XIMasterPointer) {
                 // Query pointer location and state:
                 PsychLockDisplay();
-                XIQueryPointer(dpy, indevs->deviceid, RootWindow(dpy, PsychGetXScreenIdForScreen(screenNumber)), &rootwin, &childwin, &mxd, &myd, &dxd, &dyd,
-                               &buttons_return, &modifiers_return, &group_return);
+                if (!XIQueryPointer(dpy, indevs->deviceid, mywin, &rootwin, &childwin, &dxd, &dyd, &mxd, &myd,
+                               &buttons_return, &modifiers_return, &group_return)) {
+                    // Mouse pointer is not on X-Screen of mywin. We don't have good mouse coordinates in
+                    // this case and need to fake something reasonable enough.
+                    // If an onscreen window handle was provided, simply return (mx,my) = (-1,-1). No useful
+                    // info, but at least a hit test if a mouse pointer is inside the window would fail as
+                    // expected, ie. report in "not inside window". If a screen index was provided, just use
+                    // root window relative coordinates, like we did in the past, better than nothing:
+                    if (windowRecord) {
+                        mxd = myd = -1;
+                    }
+                    else {
+                        mxd = dxd;
+                        myd = dyd;
+                    }
+                }
                 PsychUnlockDisplay();
             }
 
@@ -835,7 +847,21 @@ PsychError SCREENGetMouseHelper(void)
         else {
             // Old school core protocol query of virtual core pointer:
             PsychLockDisplay();
-            XQueryPointer(dpy, RootWindow(dpy, PsychGetXScreenIdForScreen(screenNumber)), &rootwin, &childwin, &mx, &my, &dx, &dy, &mask_return);
+            if (!XQueryPointer(dpy, mywin, &rootwin, &childwin, &dx, &dy, &mx, &my, &mask_return)) {
+                // Mouse pointer is not on X-Screen of mywin. We don't have good mouse coordinates in
+                // this case and need to fake something reasonable enough.
+                // If an onscreen window handle was provided, simply return (mx,my) = (-1,-1). No useful
+                // info, but at least a hit test if a mouse pointer is inside the window would fail as
+                // expected, ie. report in "not inside window". If a screen index was provided, just use
+                // root window relative coordinates, like we did in the past, better than nothing:
+                if (windowRecord) {
+                    mx = my = -1;
+                }
+                else {
+                    mx = dx;
+                    my = dy;
+                }
+            }
             PsychUnlockDisplay();
 
             // Copy out mouse x and y position:
@@ -928,7 +954,7 @@ PsychError SCREENGetMouseHelper(void)
 
                 // Request current keyboard state from X-Server:
                 PsychLockDisplay();
-                XQueryKeymap(dpy, keys_return);
+                XQueryKeymap(dpy, (char *) keys_return);
                 PsychUnlockDisplay();
 
                 // Request current time of query:
