@@ -59,11 +59,9 @@
 #include "Screen.h"
 
 #define MAX_SYNOPSIS_STRINGS 500
-
-//declare variables local to this file.
 static const char *synopsisSYNOPSIS[MAX_SYNOPSIS_STRINGS];
 
-void InitializeSynopsis()
+const char** InitializeSynopsis(void)
 {
     int i=0;
     const char **synopsis = synopsisSYNOPSIS;  //abbreviate the long name
@@ -254,6 +252,8 @@ void InitializeSynopsis()
     if (i > MAX_SYNOPSIS_STRINGS) {
         PrintfExit("%s: increase dimension of synopsis[] from %ld to at least %ld and recompile.",__FILE__,(long)MAX_SYNOPSIS_STRINGS,(long)i);
     }
+
+    return(synopsisSYNOPSIS);
 }
 
 PsychError PsychDisplayScreenSynopsis(void)
