@@ -43,7 +43,8 @@ psych_bool  PsychOSSetupFrameLock(PsychWindowRecordType *masterWindow, PsychWind
 psych_int64 PsychOSScheduleFlipWindowBuffers(PsychWindowRecordType *windowRecord, double tWhen, psych_int64 targetMSC, psych_int64 divisor, psych_int64 remainder, unsigned int specialFlags);
 psych_int64 PsychOSGetSwapCompletionTimestamp(PsychWindowRecordType *windowRecord, psych_int64 targetSBC, double* tSwap);
 void        PsychOSProcessEvents(PsychWindowRecordType *windowRecord, int flags);
-double      PsychOSAdjustForCompositorDelay(PsychWindowRecordType *windowRecord, double targetTime);
+double      PsychOSAdjustForCompositorDelay(PsychWindowRecordType *windowRecord, double targetTime, psych_bool onlyForCalibration);
+psych_bool  PsychOSConstrainPointer(PsychWindowRecordType *windowRecord, psych_bool constrain, PsychRectType rect);
 
 // Special query function, only defined on Linux/X11:
 int         PsychGetXScreenIdForScreen(int screenNumber);
