@@ -14,6 +14,9 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % matlab-psychtoolbox-3 instead from http://neuro.debian.net - This is more
 % convenient and will provide you with automatic updates.
 %
+% CAUTION: Psychtoolbox 3.0.14 will not work anymore with OSX versions
+% earlier than 10.11 "El Capitan".
+%
 % CAUTION: Psychtoolbox 3.0.13 will not work anymore with 32-Bit Octave-4
 % on MS-Windows, or with OSX versions earlier than 10.10 "Yosemite".
 % Psychtoolbox will likely work with versions of Microsoft Windows older
@@ -304,55 +307,61 @@ function DownloadPsychtoolbox(targetdirectory, flavor, targetRevision)
 % 10/28/15 mk  32-Bit Octave-4 support for MS-Windows reestablished.
 % 04/01/16 mk  64-Bit Octave-4 support for MS-Windows established.
 % 06/01/16 mk  32-Bit Octave-4 support for MS-Windows removed.
+% 11/16/16 mk  Don't bail if a specific 'flavor' is requested, otherwise people
+%              can not actually download older PTB versions with this downloader.
+% 06/11/18 mk  Change search order for svn executable to account for preferred location
+%              on macOS, as provided by XCode command line tools.
 
 % Flush all MEX files: This is needed at least on M$-Windows for SVN to
 % work if Screen et al. are still loaded.
 clear mex
 
-% Check if this is 32-Bit Matlab on Windows or Linux, which we don't support anymore:
-if (strcmp(computer, 'PCWIN') || strcmp(computer, 'GLNX86')) && (nargin < 2 || isempty(strfind(flavor, 'Psychtoolbox-3.0.')))
-    fprintf('Psychtoolbox 3.0.12 and later do no longer work with 32-Bit versions of Matlab.\n');
-    fprintf('You need to upgrade to a supported 64-Bit version of Octave or Matlab. 32-Bit Octave is still\n');
-    fprintf('supported on GNU/Linux.\n');
-    fprintf('If you must use a legacy 32-Bit Matlab environment, you can call this function\n');
-    fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.11'', which does support 32-Bit Matlab on Linux and Windows.\n');
-    error('Tried to setup on 32-Bit Matlab, which is no longer supported.');
-end
+if (nargin < 2 || isempty(strfind(flavor, 'Psychtoolbox-3.0.')))
+    % Check if this is 32-Bit Octave-4 on Windows, which we don't support at all:
+    if isempty(strfind(computer, 'x86_64')) && ~isempty(strfind(computer, 'mingw32'))
+        fprintf('Psychtoolbox 3.0.13 and later do no longer work with 32-Bit GNU/Octave-4 on MS-Windows.\n');
+        fprintf('You need to use 64-Bit Octave-4 if you want to use Psychtoolbox with Octave on Windows.\n');
+        fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.12'', does support 32-Bit Octave-4 on Windows.\n');
+        error('Tried to setup on 32-Bit Octave, which is no longer supported on Windows.');
+    end
 
-% Check if this is 32-Bit Octave on OSX, which we don't support anymore:
-if ~isempty(strfind(computer, 'apple-darwin')) && isempty(strfind(computer, '64'))
-    fprintf('Psychtoolbox 3.0.11 and later do no longer work with 32-Bit versions of Octave or Matlab on OSX.\n');
-    fprintf('You need to upgrade to a 64-Bit version of Octave or Matlab on OSX, which is fully supported.\n');
-    fprintf('You can also use the alternate download function DownloadLegacyPsychtoolbox() to download\n');
-    fprintf('an old legacy copy of Psychtoolbox-3.0.9, which did support 32-Bit Octave 3.2 on OSX, or use\n');
-    fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.10'', which does support 32-Bit Matlab on OSX.\n');
-    error('Tried to setup on 32-Bit Octave, which is no longer supported on OSX.');
-end
+    % Check if this is 32-Bit Matlab on Windows or Linux, which we don't support anymore:
+    if (strcmp(computer, 'PCWIN') || strcmp(computer, 'GLNX86'))
+        fprintf('Psychtoolbox 3.0.12 and later do no longer work with 32-Bit versions of Matlab.\n');
+        fprintf('You need to upgrade to a supported 64-Bit version of Octave or Matlab. 32-Bit Octave is still\n');
+        fprintf('supported on GNU/Linux.\n');
+        fprintf('If you must use a legacy 32-Bit Matlab environment, you can call this function\n');
+        fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.11'', which does support 32-Bit Matlab on Linux and Windows.\n');
+        error('Tried to setup on 32-Bit Matlab, which is no longer supported.');
+    end
 
-% Check if this is Octave-3 on Windows, which we don't support at all:
-if strcmp(computer, 'i686-pc-mingw32')
-    fprintf('Psychtoolbox 3.0.10 and later does no longer work with GNU/Octave-3 on MS-Windows.\n');
-    fprintf('You need to use 32-Bit Octave-4 if you want to use Psychtoolbox with Octave on Windows.\n');
-    fprintf('You can also use the alternate download function DownloadLegacyPsychtoolbox() to download\n');
-    fprintf('an old legacy copy of Psychtoolbox-3.0.9 which did support 32-Bit Octave 3.2 on Windows.\n');
-    error('Tried to setup on Octave, which is no longer supported on MS-Windows.');
-end
+    % Check if this is 32-Bit Octave on OSX, which we don't support anymore:
+    if ~isempty(strfind(computer, 'apple-darwin')) && isempty(strfind(computer, '64'))
+        fprintf('Psychtoolbox 3.0.11 and later do no longer work with 32-Bit versions of Octave or Matlab on OSX.\n');
+        fprintf('You need to upgrade to a 64-Bit version of Octave or Matlab on OSX, which is fully supported.\n');
+        fprintf('You can also use the alternate download function DownloadLegacyPsychtoolbox() to download\n');
+        fprintf('an old legacy copy of Psychtoolbox-3.0.9, which did support 32-Bit Octave 3.2 on OSX, or use\n');
+        fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.10'', which does support 32-Bit Matlab on OSX.\n');
+        error('Tried to setup on 32-Bit Octave, which is no longer supported on OSX.');
+    end
 
-% Check if this is 32-Bit Octave-4 on Windows, which we don't support at all:
-if isempty(strfind(computer, 'x86_64')) && ~isempty(strfind(computer, 'mingw32'))
-    fprintf('Psychtoolbox 3.0.13 and later do no longer work with 32-Bit GNU/Octave-4 on MS-Windows.\n');
-    fprintf('You need to use 64-Bit Octave-4 if you want to use Psychtoolbox with Octave on Windows.\n');
-    fprintf('DownloadPsychtoolbox() with flavor ''Psychtoolbox-3.0.12'', does support 32-Bit Octave-4 on Windows.\n');
-    error('Tried to setup on 32-Bit Octave, which is no longer supported on Windows.');
-end
+    % Check if this is Octave-3 on Windows, which we don't support at all:
+    if strcmp(computer, 'i686-pc-mingw32')
+        fprintf('Psychtoolbox 3.0.10 and later does no longer work with GNU/Octave-3 on MS-Windows.\n');
+        fprintf('You need to use 32-Bit Octave-4 if you want to use Psychtoolbox with Octave on Windows.\n');
+        fprintf('Alternatively you can download old - totally unsupported - releases of Psychtoolbox version 3.0.9.\n');
+        fprintf('See the ''Versions'' section of our website on how to do that.\n\n');
+        error('Tried to setup on Octave-3, which is no longer supported on MS-Windows.');
+    end
 
-if strcmp(computer,'MAC')
-    fprintf('This version of Psychtoolbox is no longer supported under MacOSX on the Apple PowerPC hardware platform.\n');
-    fprintf('You can get modern versions of Psychtoolbox-3 for Linux if you choose to install GNU/Linux on your PowerPC\n');
-    fprintf('machine. These are available from the GNU/Debian project and a future Ubuntu 12.10 release\n.');
-    fprintf('Alternatively you can download old - totally unsupported - releases of Psychtoolbox version 3.0.9\n');
-    fprintf('from GoogleCode by use of the alternate download function DownloadLegacyPsychtoolbox().\n\n');
-    error('Apple MacOSX on Apple PowerPC computers is no longer supported by this Psychtoolbox version.');
+    if strcmp(computer,'MAC')
+        fprintf('This version of Psychtoolbox is no longer supported under MacOSX on the Apple PowerPC hardware platform.\n');
+        fprintf('You can get modern versions of Psychtoolbox-3 for Linux if you choose to install GNU/Linux on your PowerPC\n');
+        fprintf('machine. These are available from the GNU/Debian project and a future Ubuntu 12.10 release\n.');
+        fprintf('Alternatively you can download old - totally unsupported - releases of Psychtoolbox version 3.0.9.\n');
+        fprintf('See the ''Versions'' section of our website on how to do that.\n\n');
+        error('Apple MacOSX on Apple PowerPC computers is no longer supported by this Psychtoolbox version.');
+    end
 end
 
 % Check OS
@@ -387,9 +396,8 @@ if ~isempty(v) && ~isempty(v(1).Version)
         fprintf('Please consider upgrading to a recent Matlab version or switching to GNU/Octave 3.2.x.\n');
         fprintf('Both will provide better support, performance and a richer feature set.\n\n');
         fprintf('\n\n');
-        fprintf('If you insist on use of an older Matlab version, use our alternate installer function\n\n');
-        fprintf('DownloadLegacyPsychtoolbox() \n\n');
-        fprintf('... which will allow you to retrieve a legacy version of Psychtoolbox 3.0.9 or earlier.\n');
+        fprintf('If you insist on use of an older Matlab version, see our websites ''Versions'' section\n');
+        fprintf('on how to retrieve a legacy version of Psychtoolbox 3.0.9 or earlier.\n');
         fprintf('In this case you are entirely on your own, as such versions are not supported in any way.\n\n');
         error('This Downloader does not support Matlab versions before V7.4 (R2007a) anymore.');
     end
@@ -454,9 +462,7 @@ switch (flavor)
         flavor = 'beta';
     case 'stable'
         fprintf('\n\n\nYou request download of the "stable" flavor of Psychtoolbox.\n');
-        fprintf('The "stable" flavor is no longer available, it has been renamed to "unsupported".\n');
-        fprintf('If you really want to use the former "stable" flavor, please retry the download\n');
-        fprintf('under the new name "unsupported".\n\n');
+        fprintf('The "stable" flavor is no longer available.\n');
         error('Flavor "stable" requested. This is no longer available.');
     case 'unsupported'
         % Very bad choice! Give user a chance to reconsider...
@@ -480,14 +486,9 @@ switch (flavor)
         if ~strcmp(flavor, 'beta')
             fprintf('\n\n\n\n');
             fprintf('Psychtoolbox 3.0.10 and later do no longer provide the "unsupported" flavor.\n');
-            fprintf('If you insist on use of the "unsupported" flavor, go to our GoogleCode site ...\n\n');
-            fprintf('http://code.google.com/p/psychtoolbox-3/ \n\n');
-            fprintf('... and get yourself an older version of DownloadPsychtoolbox.m which will allow you\n');
-            fprintf('to retrieve such a legacy version of Psychtoolbox 3.0.9 or earlier. In that case,\n');
-            fprintf('you are entirely on your own, as such versions are not supported in any way.\n\n');
             error('This Downloader does not support the "unsupported" aka "stable" flavor anymore.');
         end
-        
+
     otherwise
         fprintf('\n\n\nHmm, requested flavor is the unusual flavor: %s\n',flavor);
         fprintf('Either you request something exotic, or you made a typo?\n');
@@ -523,10 +524,6 @@ else
     if IsOSX
         svnpath = '';
 
-        if isempty(svnpath) && exist('/opt/subversion/bin/svn', 'file')
-            svnpath = '/opt/subversion/bin/';
-        end
-
         if isempty(svnpath) && exist('/usr/bin/svn','file')
             svnpath='/usr/bin/';
         end
@@ -543,6 +540,10 @@ else
             svnpath = '/opt/local/bin/';
         end
 
+        if isempty(svnpath) && exist('/opt/subversion/bin/svn', 'file')
+            svnpath = '/opt/subversion/bin/';
+        end
+
         if isempty(svnpath)
             fprintf('The Subversion client "svn" is not in one of its expected\n');
             fprintf('locations for Mac OSX  on your disk. Please download and\n');
@@ -783,6 +784,11 @@ end
 if err
     fprintf('Sorry, the download command "CHECKOUT" failed with error code %d: \n',err);
     fprintf('%s\n',result);
+    if IsOSX
+        fprintf('If the error output above contains the text ''SSL handshake failed: SSL error: tlsv1 alert protocol version''\n');
+        fprintf('then your svn command line client is too old. Get a more recent one, e.g., from XCode developer tools, or the\n');
+        fprintf('HomeBrew package manager or such.\n');
+    end
     fprintf('The download failure might be due to temporary network or server problems. You may want to try again in a\n');
     fprintf('few minutes. It could also be that the subversion client was not (properly) installed. On Microsoft\n');
     fprintf('Windows you will need to exit and restart Matlab or Octave after installation of the Subversion client. If that\n');
