@@ -109,8 +109,40 @@ try
             % Get current path:
             path = getenv('PATH');
 
-            % Prepend sdkroot to path:
-            newpath = [sdkroot ';' path];
+            if ~IsOctave || IsGUI
+                % Matlab, or Octave in GUI mode: Prepend sdkroot to path:
+                newpath = [sdkroot ';' path];
+            else
+                % Octave CLI: Do not add GStreamer runtime, so we can build
+                % our mex files without interference of GStreamer's libstdc++.6.dll:
+                newpath = path;
+            end
+
+            % For Octave-4.2 on Windows, also need to prepend path to portaudio dll
+            % to library path, otherwise PsychPortAudio won't load:
+            driverloadpath = [PsychtoolboxRoot 'PsychSound'];
+            newpath = [driverloadpath ';' newpath];
+
+            % For moglcore we need to prepend the path to freeglut.dll:
+            if Is64Bit
+                % Need 64-Bit freeglut.dll:
+                driverloadpath = [PsychtoolboxRoot 'PsychOpenGL/MOGL/core/x64'];
+            else
+                % Need 32-Bit freeglut.dll:
+                driverloadpath = [PsychtoolboxRoot 'PsychOpenGL/MOGL/core'];
+            end
+            newpath = [driverloadpath ';' newpath];
+
+            % For PsychHID we need to prepend the path to libusb-1.0.dll:
+            if Is64Bit
+                % 64-Bit version of libusb.dll
+                driverloadpath = [PsychtoolboxRoot 'PsychContributed' filesep 'x64' filesep];
+            else
+                % 32-Bit version of libusb.dll
+                driverloadpath = [PsychtoolboxRoot 'PsychContributed' filesep ];
+            end
+            newpath = [driverloadpath ';' newpath];
+
             setenv('PATH', newpath);
 
             % Also store sdkroot in a separate environment variable, to be used
